@@ -1,10 +1,6 @@
-#!/usr/bin/env python
+# -*- coding: utf-8 -*-
 #
-# $Revision: 1.36 $
-# $Date: 2005/08/15 21:19:55 $
-# $Author: dwelch $
-#
-# (c) Copyright 2003-2004 Hewlett-Packard Development Company, L.P.
+# (c) Copyright 2003-2006 Hewlett-Packard Development Company, L.P.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,7 +19,6 @@
 # Author: Don Welch
 #
 
-
 # Local
 from g import *
 from codes import *
@@ -36,7 +31,7 @@ def AlignType1(dev, loadpaper_ui): # Auto VIP
     ok = loadpaper_ui()
     if ok:
         dev.writeEmbeddedPML(pml.OID_AUTO_ALIGNMENT,
-                              pml.AUTO_ALIGNMENT)
+                              pml.AUTO_ALIGNMENT, style=0)
 
     return ok
 
@@ -1213,3 +1208,15 @@ def colorCalType4Phase3(dev):
                          pml.PRINT_INTERNAL_PAGE_COLOR_PALETTE_CMYK_PAGE)
 
 
+
+def colorCalType5(dev, loadpaper_ui):
+    if loadpaper_ui():
+        dev.printData("""\x1b%-12345X@PJL ENTER LANGUAGE=PCL3GUI\n\x1bE\x1b%Puifp.multi_button_push 20;\nudw.quit;\x1b*rC\x1bE\x1b%-12345X""")
+    
+def linefeedCalType1(dev, loadpaper_ui):
+    if loadpaper_ui():
+        dev.printData("""\x1b%-12345X@PJL ENTER LANGUAGE=PCL3GUI\n\x1bE\x1b%Puifp.multi_button_push 3;\nudw.quit;\x1b*rC\x1bE\x1b%-12345X""")
+    
+def printQualityDiagType1(dev, loadpaper_ui):
+    if loadpaper_ui():
+        dev.printData("""\x1b%-12345X@PJL ENTER LANGUAGE=PCL3GUI\n\x1bE\x1b%Puifp.multi_button_push 14;\nudw.quit;\x1b*rC\x1bE\x1b%-12345X""")
